import * as dotenv from 'dotenv';
import {Guid} from 'guid-typescript';
import {Event} from './event.interface';
import {NachklangCalendarDB} from '../Calendar.db';


dotenv.config();

/**
 * Returns all events for the given calendar
 * @param calendarId The calendar Id
 */
export const getAllEvents = async (calendarId: number): Promise<Event[]> => {
	let conn = await NachklangCalendarDB.getConnection();
	let eventRows: Event[] = [];
	try {
		const eventsQuery = 'SELECT * FROM events WHERE calendar_id = ?';
		const eventsRes = await conn.query(eventsQuery, calendarId);

		for (let row of eventsRes) {
			eventRows.push(row);
		}

		return eventRows;
	} catch (err) {
		throw err;
	} finally {
		// Return connection
		await conn.end();
	}
};

/**
 * Create the given event in the database
 * @param event The event to create
 */
export const createEvent = async (event: Event): Promise<number> => {
	let conn = await NachklangCalendarDB.getConnection();
	try {
		let eventUUID = Guid.create().toString();
		const eventsQuery = 'INSERT INTO events (calendar_id, uuid, name, description, start_datetime, end_datetime, location, created_by, url) VALUES (?,?,?,?,?,?,?,?,?) RETURNING event_id';
		const eventsRes = await conn.query(eventsQuery, [event.calendar_id, eventUUID, event.name, event.description, event.start_datetime, event.end_datetime, event.location, event.created_by, event.url]);

		return eventsRes[0].event_id;
	} catch (err) {
		throw err;
	} finally {
		// Return connection
		await conn.end();
	}
};

/**
 * Update the given event in the database
 * @param event The event to update
 */
export const updateEvent = async (event: Event): Promise<boolean> => {
	let conn = await NachklangCalendarDB.getConnection();
	try {
		let eventUUID = Guid.create().toString();
		const eventsQuery = 'UPDATE events SET name = ?, description = ?, start_datetime = ?, end_datetime = ?, location = ?, created_by = ?, url = ? WHERE event_id = ?';
		const eventsRes = await conn.query(eventsQuery, [event.name, event.description, event.start_datetime, event.end_datetime, event.location, event.created_by, event.url, event.event_id]);

		console.log(eventsRes);

		return eventsRes.affectedRows === 1;
	} catch (err) {
		throw err;
	} finally {
		// Return connection
		await conn.end();
	}
};
