import {Event} from './event.interface';

export const convertToIcal = async (events: Event[]): Promise<string> => {
	try {
		let ical: iCalFile = {body: []};
		generateHeaderInfo(ical);
		generateFooterInfo(ical);
		for (let event of events) {
			addEventToFile(ical, event);
		}

		return serializeIcalFile(ical);
	} catch (err) {
		throw err;
	}
};

const serializeIcalFile = (ical: iCalFile): string => {
	let returnString = '';

	returnString += ical.header;
	for (let event of ical.body) {
		returnString += serializeIcalEvent(event);
	}
	returnString += ical.footer;

	return returnString;
};

const serializeIcalEvent = (icalevent: iCalEvent): string => {
	let returnString = '';

	returnString += icalevent.header;
	returnString += 'UID:' + icalevent.uid;
	returnString += 'DTSTAMP:' + icalevent.created;
	returnString += 'ORGANIZER:' + icalevent.organizer;
	returnString += 'DTSTART;TZID=Europe/Berlin:' + icalevent.start;
	returnString += 'DTEND;TZID=Europe/Berlin:' + icalevent.end;
	returnString += 'SUMMARY:' + icalevent.summary;
	returnString += 'DESCRIPTION:' + icalevent.description;
	returnString += 'LOCATION:' + icalevent.location;
	returnString += 'URL:' + icalevent.url;
	returnString += icalevent.footer;

	return returnString;
};


const generateHeaderInfo = (ical: iCalFile) => {
	ical.header = 'BEGIN:VCALENDAR\n' +
		'VERSION:2.0\n' +
		'PRODID:-//Nachklang e.V./Nachklang Calendar//NONSGML v1.0//EN\n' +
		'CALSCALE:GREGORIAN\n' +
		'BEGIN:VTIMEZONE\n' +
		'TZID:Europe/Berlin\n' +
		'LAST-MODIFIED:20201011T015911Z\n' +
		'TZURL:http://tzurl.org/zoneinfo-outlook/Europe/Berlin\n' +
		'X-LIC-LOCATION:Europe/Berlin\n' +
		'BEGIN:DAYLIGHT\n' +
		'TZNAME:CEST\n' +
		'TZOFFSETFROM:+0100\n' +
		'TZOFFSETTO:+0200\n' +
		'DTSTART:19700329T020000\n' +
		'RRULE:FREQ=YEARLY;BYMONTH=3;BYDAY=-1SU\n' +
		'END:DAYLIGHT\n' +
		'BEGIN:STANDARD\n' +
		'TZNAME:CET\n' +
		'TZOFFSETFROM:+0200\n' +
		'TZOFFSETTO:+0100\n' +
		'DTSTART:19701025T030000\n' +
		'RRULE:FREQ=YEARLY;BYMONTH=10;BYDAY=-1SU\n' +
		'END:STANDARD\n' +
		'END:VTIMEZONE\n';
};

const generateFooterInfo = (ical: iCalFile) => {
	ical.footer = 'END:VCALENDAR';
};

const addEventToFile = (ical: iCalFile, event: Event) => {
	ical.body.push(createIcalEvent(event));
};

const createIcalEvent = (event: Event): iCalEvent => {
	return {
		header: 'BEGIN:VEVENT\n',
		uid: event.uuid + '\n',
		created: formatDate(event.created_date) + 'Z\n',
		organizer: event.created_by + '\n',
		start: formatDate(event.start_datetime) + '\n',
		end: formatDate(event.end_datetime) + '\n',
		summary: event.name + '\n',
		description: event.description + '\n',
		location: event.location + '\n',
		url: event.url + '\n',
		footer: 'END:VEVENT\n'
	};
};

const formatDate = (date: Date): string => {
	let returnString = '';

	returnString += date.getFullYear();
	returnString += (date.getMonth() + 1).toString().padStart(2, '0'); // +1 Because JS sucks
	returnString += date.getDate().toString().padStart(2, '0');
	returnString += 'T';
	returnString += date.getHours().toString().padStart(2, '0');
	returnString += date.getMinutes().toString().padStart(2, '0');
	returnString += date.getSeconds().toString().padStart(2, '0');

	return returnString;
};

export interface iCalFile {
	header?: string;
	body: iCalEvent[];
	footer?: string;
}

export interface iCalEvent {
	header: string;
	uid: string;
	created: string;
	organizer: string;
	start: string;
	end: string;
	summary: string;
	description: string;
	location: string;
	url: string;
	footer: string;
}
